#!/usr/bin/python

# Viscosity DNS Support Script
# http://www.viscosityvpn.com

import os, sys, re
from subprocess import *

# Update path for running instance to make sure we can find the programs
os.environ["PATH"] = '/bin:/sbin:/usr/sbin:/usr/bin'

def usage(msg):
    print >> sys.stderr, '%s: %s' % (sys.argv[0], msg)
    sys.exit(1)

# Get command line args
try:
#    tun_dev, tun_mtu, link_mtu, ifconfig_local_ip, ifconfig_remote_ip, \
    tun_dev, tun_mtu, link_mtu = sys.argv[2:]
except: usage('expected arguments from OpenVPN')

if sys.argv[4] not in ('init', 'down'): usage('expected "init" or "down"')
going_up = sys.argv[4] == 'init'

networkInt = "NULL"

# Get DHCP options directly from OpenVPN's tap adapter
# Get first tap interface
cmd = '''(scutil | grep "tap0" | sed -e 's/.*: //')<< EOF
open
get State:/Network/Interface
d.show
quit
EOF'''
p = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
networkInt = p.stdout.read()[:-1]

# Bail if we don't have any tap interfaces
if networkInt != "tap0":
	file = open('/Library/Logs/Viscosity-DNSUP.log', 'w')
	file.write("tap0 not found.")
	file.close()
	sys.exit(0)

# Get DNS and domains
dnsservers = []
domains = []

try:
	# Get DHCP tap0 Domain record
	cmd = '''(scutil)<< EOF
	open
	get State:/Network/Service/DHCP-%s/DNS
	d.show
	quit
	EOF''' % networkInt
	p = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
	lines = p.stdout.read().split('\n');
	data = [ x for x in lines if 'DomainName :' in x]
	domains = [ domain.split(':')[-1].strip() for domain in data ]
	data = [ x for x in lines if x.split(':')[0].strip().isdigit()]
	dnsservers = [ ns.split(':')[-1].strip() for ns in data]

except:
	import traceback
	file = open('/Library/Logs/Viscosity-DNSUP.log', 'w')
	file.write(traceback.format_exc())
	file.close()


try:
	# Get DHCP tap0 IP address
	cmd = '''(scutil)<< EOF
	open
	get State:/Network/Service/DHCP-%s/IPv4
	d.show
	quit
	EOF''' % networkInt
	p = Popen(cmd, shell=True, stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True)
	lines = p.stdout.read().split('\n');
	data = [ x for x in lines if 'Addresses :' in x]
	local_ip = [ ip.split(':')[-1].strip() for ip in data ]

except:
	import traceback
	file = open('/Library/Logs/Viscosity-DNSUP.log', 'w')
	file.write(traceback.format_exc())
	file.close()

# Don't keep going if we don't have any DNS servers
if len(dnsservers) == 0:
	file = open('/Library/Logs/Viscosity-DNSUP.log', 'w')
	file.write("No DNS servers parsed correctly from scutil.")
	file.close()
	sys.exit(0)

def get_sc_store():
	return SCDynamicStoreCreate(None, "OpenVPN", handle_sc_event, None)

def handle_sc_event(store, changed_keys, info):
	for key in changed_keys:
          SC_HANDLERS[key](key=key, info=info)

try:
	# Set dummy domain if none specified
	if len(domains) == 0:
		domains.append('viscosity.default.search.domain')

	# for Mac OS X Tiger and later, use supplemental match domains
	# see this thread for more information:
	# <http://lists.apple.com/archives/Macnetworkprog/2005/Jun/msg00011.html>
	from SystemConfiguration import *
	SC_IPV4_PATH, SC_DNS_PATH = ['/'.join((kSCDynamicStoreDomainState,
				   kSCCompNetwork, kSCCompService,
				   networkInt, ent))
				for ent in (kSCEntNetIPv4, kSCEntNetDNS)]
	store = get_sc_store()
	if going_up:

#	  ipv4 = {kSCPropNetIPv4Addresses: [local_ip],
#	  	kSCPropNetIPv4DestAddresses: [remote_ip],
#	 	kSCPropInterfaceName: networkInt}
	  dns = {kSCPropNetDNSServerAddresses: dnsservers,
		kSCPropNetDNSSupplementalMatchDomains: domains}
	  SCDynamicStoreAddValue(store, SC_DNS_PATH, dns)
#	  store.setValue_forKey_(ipv4, SC_IPV4_PATH)
#	  store.setValue_forKey_(dns, SC_DNS_PATH)
	resolv_file_path = '/var/run/resolv.conf'
	resolv_file = file(resolv_file_path, 'r+')
	resolv_lines = resolv_file.readlines()

	START_SKIP = '# begin %s OpenVPN tunnel modifications' % networkInt
	END_SKIP = '# end %s OpenVPN tunnel modifications' % networkInt
	domains = ['search'] + domains

	in_skip = False
	search_line = ' '.join(domains)

	resolv_file.seek(0)
	resolv_file.truncate()
	for line in resolv_lines:
		if in_skip:
			if re.match(END_SKIP, line):
				in_skip = False
			continue
		else:
			if re.match(START_SKIP, line):
				in_skip = True
				continue
		if line == '\n':
			continue
		if re.match('nameserver ', line):
			resolv_file.write('# *#* ')
		if re.match('search ', line):
			resolv_file.write('# *#* ')
			search_line = ' '.join(domains + \
				   [domain for domain in line[:-1].split(' ')
					if domain not in domains])
		resolv_file.write(line)

	print >> resolv_file
	print >> resolv_file, START_SKIP
	for nameserver in dnsservers:
		print >> resolv_file, 'nameserver', nameserver
	print >> resolv_file, search_line
	print >> resolv_file, END_SKIP

	resolv_file.close()
	
except:
	import traceback
	file = open('/Library/Logs/Viscosity-DNSUP.log', 'w')
	file.write(traceback.format_exc())
	file.close()
